local waves = {}
local explosions = {}
local circleX, circleY, circleRad = 0, 0, 0

local function drawArc(x, y, r, angle1, angle2, segments)
	local i = angle1
	local j = 0
	local step = math.pi*2 / segments
	    
	while i < angle2 do
	  	j = angle2 - i < step and angle2 or i + step
	  	love.graphics.line(x + (math.cos(i) * r), y - (math.sin(i) * r), x + (math.cos(j) * r), y - (math.sin(j) * r))
	  	i = j
	end
end

local function stencil()
   love.graphics.circle("fill", circleX, circleY, circleRad, 50)
end

function addWave(ang)
	table.insert(waves, {a = 0, rr = ang, v = 80, d = 32, r = 32}) -- v is speed of spread
end

function addExplosion(x, y, rad, vel)
	local id = #explosions + 1
	table.insert(explosions, {x = x, y = y, r = 0})

	local i = explosions[#explosions]
	flux.to(i, (rad / vel), {r = rad}):after(i, (rad / (vel*2)), {r = 0}):delay(0.5):oncomplete( function() table.remove(explosions, id) end )
end

function drawExplosions(c)
	local col = c or white
	love.graphics.setColor(col)
	love.graphics.setLineWidth(2)

	for i,v in ipairs(explosions) do
		love.graphics.circle("line", v.x, v.y, v.r, 50)
	end

	love.graphics.setColor(black)
	for i,v in ipairs(explosions) do
		love.graphics.circle("fill", v.x, v.y, v.r, 50)
	end
	love.graphics.setLineWidth(1)
end

function drawForceField(x, y, layer, c)
	local col = c or red
	--[[love.graphics.setColor(lightred[1], lightred[2], lightred[3], 127)
	love.graphics.circle("fill", x, y, 32, 50)]]--
	love.graphics.setColor(col[1], col[2], col[3], 200)
	love.graphics.setLineWidth(2)
	--love.graphics.circle("line", x, y, 32, 50)

	for i,v in ipairs(waves) do
		love.graphics.push()
		love.graphics.translate(x, y)
		love.graphics.rotate(v.rr)
		love.graphics.translate(-x, -y)

		local xx = math.cos(v.a)*v.d + x
		local yy = math.sin(v.a)*v.d + y

		local xx2 = math.cos(v.a)*(v.d+24) + x
		local yy2 = math.sin(v.a)*(v.d+24) + y

		local xx3 = math.cos(v.a)*(v.d-24) + x
		local yy3 = math.sin(v.a)*(v.d-24) + y

		local a = math.acos(v.d/32)

		circleX, circleY, circleRad = xx, yy, math.sin(v.a + a)*v.r
		love.graphics.stencil(stencil, "replace", 1)
		love.graphics.setStencilTest("greater", 0)

		if layer == "front" then
			love.graphics.circle("line", xx2, yy2, math.dist(xx2, yy2, xx, math.sin(v.a + a)*v.r + yy), 50)
		elseif layer == "back" then
			love.graphics.circle("line", xx3, yy3, math.dist(xx3, yy3, xx, math.sin(v.a + a)*v.r + yy), 50)
		end

		if layer == "front" then
			v.d = v.d - love.timer.getDelta()*v.v
		end

		if v.d < -v.r then table.remove(waves, i) end
		
		--love.graphics.setStencil()
		love.graphics.setStencilTest()
		love.graphics.pop()
	end

	love.graphics.setLineWidth(1)
end

-- Returns the distance between two points.
function math.dist(x1,y1, x2,y2) return ((x2-x1)^2+(y2-y1)^2)^0.5 end